#!/usr/bin/perl

package display_map;

use GD;
use FileHandle;


sub new
{
    my ($this, $display, $cfg) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $cfg = "$main::root/adm/display_map.cfg" unless( $cfg );
    $self->{"cfg"} = $cfg;
    $self->{"display"} = $display;
    print "display_map::new() created object for $cfg\n" if( $main::dl>4 );
    return $self;
}


sub set_display {
    my( $self, $display ) = @_;

    $self->{"display"} = $display;
}


sub create {
    my ( $self ) = @_;
    my $cfg = FileHandle->new;
    my $seq = FileHandle->new;
    my ($cmd, $args, @args);
    my( $image, %stats, $map, $i, %colors, %positions );

    print "display_map::create()\n" if( $main::dl>4 );
    open( $cfg, "<".$self->{"cfg"} ) || return;
    if( open( $seq, "<".$self->{"cfg"}.".seq" ) ) {
	$_ = <$seq>;
	chomp;
	$self->{"alt"} = $_+1;
	close $seq;
    }
    open( $seq, ">".$self->{"cfg"}.".seq" );
    seek( $seq, 0, 0 );
    print $seq ($self->{"alt"}+1)."\n";
    close $seq;
    $self->{"map"} = $map = [];
    while( <$cfg> ) {
	print "display_map::read line: $_" if( $main::dl>7 );
	chomp;
	next if( /^#/ || ! $_);
	if( /^(template|red|green|yellow|purple|clear|blue|graphics|at|dump|name|line)[\s\t]+(.*)/ ) {
	    ($cmd, $args) = ($1,$2);
	    @args = split( /[\s\t]+/, $args );
	    if( $cmd eq "template" ) {
		print "display_map::create: read image from $args[0]\n" if( $main::dl>5 );
		$image = $self->fromgif( $args[0] );
		last unless( $image );
		$image->interlaced(1);
		$colors{"red"} = $image->colorAllocate(255,0,0);
		$colors{"green"} = $image->colorAllocate(0,255,0);
		$colors{"yellow"} = $image->colorAllocate(255,255,0);
		$colors{"purple"} = $image->colorAllocate(0,255,255);
		$colors{"clear"} = $image->colorAllocate(228,228,228);
		$colors{"blue"} = $image->colorAllocate(0,0,255);
	    }
	    elsif( grep( $cmd eq $_, "red", "green", "purple", "yellow", "blue", "clear" ) ) {
		$stats{$cmd} = $self->fromgif( $args[0], 1 );
		last unless( $stats{$cmd} );
		print "display_map::create: read status image $cmd from $args[0]\n" if( $main::dl>5 );
	    }
	    elsif( $cmd eq "dump" ) {
		my( $fh, $file, $alt, $former );

		$fh = FileHandle->new();
		$file = $args[0];
		$alt = $self->{"alt"};
		if( ! ($file =~ /\.(jpg|jpeg|gif|png)$/) ) {
		    $file .= ".png";
		}
		for( $i=$alt-2; $i<=$alt; $i++ ) {
 		    $former = $file;
		    $former =~ s/\./$i\./;
		    unlink( $main::root."/".$former );
	        }
		$alt+=2;
		$file =~ s/\./$alt\./;
		if( ($file =~ /\.(jpg|jpeg)$/) && ! defined &GD::Image::jpeg ) {
		    $file =~ s/\.jp.{0,1}g$/\.png/;
		}
		if( ($file =~ /\.png$/) && ! defined &GD::Image::png ) {
		    $file =~ s/\.png$/\.gif/;
		}
		if( ($file =~ /\.gif$/) && ! defined &GD::Image::gif ) {
		    $file =~ s/\.gif$/\.png/;
		}
		open( $fh, ">$main::root/".$file ) || last;
		binmode $fh;
		($file =~ /\.png$/) && (print $fh $image->png);
		($file =~ /\.gif$/) && (print $fh $image->gif);
		($file =~ /\.(jpg|jpeg)$/) && (print $fh $image->jpeg);
		close( $fh );
		$self->{"recent_file"} = $file;
		print "display_map::create: dumped image to $args[0]\n" if( $main::dl>5 );
	    }
	    elsif( $cmd eq "at" ) {
		my( $pos, $group, $ins ) = @args;
		my( $x, $y ) = $self->coord( $pos, %positions );
		my( $w, $h, $status );

		$status = $self->{"display"}->group_status( $group );
		if( $ins = $stats{$status} ) {
		    ($w,$h) = $ins->getBounds();
		    $image->copy($ins,$x-$w/2,$y-$h/2,0,0,$w,$h);
		    push( @$map, [ $x, $y, $w, $h, $group ] );
		}
		print "display_map::create: inserted status $status for $group at $pos\n" if( $main::dl>5 );
	    }
	    elsif( $cmd eq "name" ) {
		my( $pos, $name ) = @args;
		my( $x, $y ) = $self->coord( $pos, %positions );
		$positions{$name} = [$x,$y];
	    }
	    elsif( $cmd eq "line" ) {
		my( $pos1, $pos2, $group ) = @args;
		my( $status );

		$status = $self->{"display"}->group_status( $group );
		if( defined $colors{$status} ) {
		    $image->line( $self->coord( $pos1, %positions ), $self->coord( $pos2, %positions ), $colors{$status} );
		    print "display_map::create: lined status $status for $group\n" if( $main::dl>5 );
		}
	    }
	}
    }
    close( $cfg );
}

		

sub coord {
    my( $self, $pos, %positions ) = @_;

    if( defined $positions{$pos} ) {
	return( $positions{$pos}->[0], $positions{$pos}->[1] );
    }
    return( split( /[,;:,\.]/, $pos ) );
}



sub file {
    my( $self ) = @_;

    return $self->{"recent_file"};
}


sub map {
    my( $self ) = @_;

    return( $self->{"map"} ) if( $self->{"map"} );
    return( [] );
}


sub updated {
    my( $self ) = @_;
}



sub fromgif {
    my( $self, $file, $transparent ) = @_;
    my( $fh, $image );

    print "display_map::fromgif($file)\n" if( $main::dl>5 );
    if( defined &GD::Image::png ) {
	# "new" GD module
	$image = GD::Image->new("$main::root/".$file);
    }
    else {
	open( _GIF, "<$main::root/".$file ) || return(undef);
	binmode _GIF;
	$image = GD::Image->newFromGif(_GIF);
	close( _GIF );
    }
    ($image = GD::Image->new(1,1)) unless( $image );
    if( ($image->transparent < 0) && $transparent ) {
	$image->transparent($image->colorClosest(0,0,0));
    }
    return $image;
}


1;
